// @flow
import React, { Component } from 'react'
import {
  Text, FlatList, View, Platform, Dimensions,
} from 'react-native'
import styleConstructor from './Slider.style'

type Props = {
  title: Text,
  data: array,
  rtl: boolean,
  itemCount: number,
  renderItem: Function,
  separatorWidth: number,
  AIVisibleArea: number,
  paginationCount: number,
  textTitleStyle: object,
  containerStyle: object,
  style: object
};
const SCREEN = Dimensions.get('screen')

export default class Slider extends Component<Props> {
  static defaultProps = {
    data: [],
    rtl: true,
    itemCount: 1,
    renderItem: item => item,
    separatorWidth: 8,
    AIVisibleArea: 8,
    paginationCount: 1,
    textTitleStyle: {},
    containerStyle: {},
    style: { height:200 },
  }

  constructor(props: Props) {
    super(props)
    this.style = styleConstructor()
    const self = this
    self.keyExtractor = this.keyExtractor.bind(this)
    self.renderHeader = this.renderHeader.bind(this)
    self.renderItem = this.renderItem.bind(this)
  }

  keyExtractor(item) {
    return item.id
  }

  renderItem = ({ item, index }) => {
    const {
      renderItem, separatorWidth, itemCount, AIVisibleArea, rtl,style
    } = this.props
    const itemWidth = (SCREEN.width - AIVisibleArea * 2 - separatorWidth * (itemCount + 1)) / itemCount
    return (
      <View
        style={{
          width: itemWidth,
          height: style.height,
          marginLeft: rtl ? separatorWidth : 0,
          marginRight: rtl ? 0 : separatorWidth,
        }}
      >
        {renderItem({ item, index })}
      </View>
    )
  }

  renderHeader() {
    const {
      title, textTitleStyle, separatorWidth, AIVisibleArea,
    } = this.props
    const { titleContainer, titleStyle } = this.style
    return (
      <View
        style={{
          alignItems: 'flex-end',
          marginRight: separatorWidth + AIVisibleArea,
          marginBottom: 17.5,
        }}
      >
        <Text style={[titleContainer, textTitleStyle]}>{title}</Text>
      </View>
    )
  }

  render() {
    const {
      data,
      rtl,
      separatorWidth,
      itemCount,
      AIVisibleArea,
      title,
      paginationCount,
      style,
      containerStyle,
    } = this.props
    const itemWidth = (SCREEN.width - AIVisibleArea * 2 - separatorWidth * (itemCount + 1)) / itemCount
    return (
      <View style={[{ flexDirection: 'column' }, containerStyle]}>
        {title && this.renderHeader()}
        <FlatList
          style={style}
          rtl={rtl}
          horizontal
          data={data}
          contentContainerStyle={{ paddingHorizontal: AIVisibleArea + separatorWidth }}
          showsHorizontalScrollIndicator={false}
          renderItem={this.renderItem}
          ref={ref => (this.FlatList = ref)}
          keyExtractor={this.keyExtractor}
          snapToInterval={paginationCount * (itemWidth + separatorWidth)}
          getItemLayout={(data, index) => ({
            length: itemWidth,
            offset: itemWidth * index,
            index,
          })}
          snapToAlignment={Platform.OS === 'ios' ? 'start' : 'center'}
          decelerationRate={Platform.select({android:"normal",ios:"fast"})}
          pagingEnabled={Platform.OS !== 'ios'}
        />
      </View>
    )
  }
}
